/*
 *  udebootstrap - micro debootstrapping code.
 *  Copyright (C) 2002 Junichi Uekawa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Depends: tar, ar, zcat, sh
 */

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include "config.h"


void
error_fail(const char * s)
{
  fprintf(stderr, "Error: %s\n", s);
  exit (1);
}

int
extract_package(const char *target_dir, const char * extract_package_name)
{
  char * cwd = get_current_dir_name();
  char * udeb_name = canonicalize_file_name(extract_package_name);
  char * buf ;
  

  if (!cwd) error_fail("cannot get cwd in extract_package");
  if (!udeb_name) error_fail("cannot get udeb name in extract_package");
  
  chdir (target_dir);
  printf("  Extracting [%s] ... ", udeb_name);
  fflush(stdout);

  asprintf (&buf, "ar -p %s data.tar.gz | zcat | tar -xf - ", udeb_name);
  if (system (buf))
    {
      printf("failed\n");
      error_fail("extract_package failed\n");
    }
  else
    printf("done\n");

  chdir (cwd);			/* revert to previous dir. */
  
  free(buf);
  free(cwd);
  free(udeb_name);
  return 0;
}

/**
   Files are probably found in ./udeb directory, let us go and iterate through all of them, 
   and extract it to ./target
 */
int 
iterate_files_in_udeb_dir(const char * root_path, const char * path_to_udebs)
{
  struct dirent * d;
  DIR * dir;
  char * buf;
  
  printf ("Extracting udebs from %s to %s\n", path_to_udebs, root_path);
  dir=opendir(path_to_udebs);
  
  while ((d=readdir(dir))!=NULL)
    {
      int l = strlen(d->d_name);
      if (
	  l > 5 &&
	  (((d->d_name[l-5] == '.') &&
	    (d->d_name[l-4] == 'u') &&
	    (d->d_name[l-3] == 'd') &&
	    (d->d_name[l-2] == 'e') &&
	    (d->d_name[l-1] == 'b')) ||
	   ((d->d_name[l-4] == '.') &&
	    (d->d_name[l-3] == 'd') &&
	    (d->d_name[l-2] == 'e') &&
	    (d->d_name[l-1] == 'b'))))
	{
	  asprintf (&buf, "%s/%s", path_to_udebs, d->d_name);
	  extract_package(root_path, buf);
	  free(buf);
	}
    } 
  closedir(dir);
  return 0;
}

int
main(int ac, char ** av)
{
  printf ("udebootstrap Version %s\n\n"
	  "Micro-Debian-Installer bootstrapping application\n"
	  "Proof-of-concept implementation by Junichi Uekawa\n\n"
	  ,
	  VERSION);
  iterate_files_in_udeb_dir("test", "./udeb/");
  
  if (!chroot ("test"))
    {
      chdir("/");
      execl("/bin/sh", "/bin/sh", NULL);
      error_fail("execl failed");
    }
  else
    error_fail("chroot failed");
  
  return 0;			/* this should never really return.. should it ? */
}

